#!/bin/bash

if [ "$EUID" -ne 0 ]; then
	echo "The script needs root privileges. Exiting."
	exit 1
fi

msg () {
  echo "cc-auto-release-update: $1" | tee /dev/kmsg
}

FW_UPDATE_DIR="fw_update"
RESCUE_UPDATE_DIR="rescue_update"
FW_UPDATE_TARGET="/opt"
FW_UPDATE_TARGET_PATH="${FW_UPDATE_TARGET}/${FW_UPDATE_DIR}"

case $# in
	1)
		SRC_DIR="$1"
		USB_DIR=""
		UPDATE_DONE_PATH="${SRC_DIR}/update-done"
		;;
	2)
		SRC_DIR="$1"
		USB_DIR="$2"
		UPDATE_DONE_PATH="${USB_DIR}/update-done"
		;;
	*)
		msg "Error. Script must be called with following arguments:"
		msg "  <SRC_DIR> - path to the installation files, whether USB stick or another location"
		msg "  <USB_DIR> - optional - should point to the USB stick location in case the installation path is somewhere else"
		exit 1
		;;
esac

#
# Sanity check the directories provided
#
if [ ! -d "$SRC_DIR" ]; then
	msg "Installation source directory does not exists. Exiting."
	exit 1
fi

if [ ! -z "$USB_DIR" ] && [ ! -d "$USB_DIR" ]; then
	msg "USB directory does not exists. Exiting."
	exit 1
fi

#
# To prevent accidental updates, use this stamp file for it.
#
if [ -e "${SRC_DIR}/update-done" ] ; then
        msg "update-done flag exists, not running update"
        exit 0
fi

if [ ! -z "$USB_DIR" ] && [ -e "${USB_DIR}/update-done" ]; then
	msg "update-done flag exists in the USB, not running update"
	exit 0
fi

echo $(date +%s) > "$UPDATE_DONE_PATH" && sync

ccsettingsconsole --led --color=green --frequency=7 --dutycycle=50

# Check if we are going to upgrade rescue or main
if [ -e "${SRC_DIR}/${RESCUE_UPDATE_DIR}" ] ; then
	#
	# RESCUE UPDATE
	#
	if [ -z "$USB_DIR"]; then
		# Regular update from usb-stick - no USB_DIR provided
		# Run update from /tmp - directory after unmounting USB-stick.
		cp -a "${SRC_DIR}/${RESCUE_UPDATE_DIR}" /tmp && \
			cd /tmp/rescue_update && sync && \
			sleep 2 && umount "$SRC_DIR" && \
			./fullup.sh -f -s -3 && \
			ccsettingsconsole --led --color=green --frequency=1 --dutycycle=50 && \
			reboot &
	else
		# Update from secure package
		# Run update directly from given location
		cd "${SRC_DIR}/${RESCUE_UPDATE_DIR}" && \
			./fullup.sh -f -s -3 && \
			ccsettingsconsole --led --color=green --frequency=1 --dutycycle=50 && \
			reboot
	fi
elif [ -e "${SRC_DIR}/${FW_UPDATE_DIR}" ] ; then
	#
	# MAIN UPDATE
	#
	msg "Copying FW update to $FW_UPDATE_TARGET_PATH"
	cp -a "${SRC_DIR}/${FW_UPDATE_DIR}" "$FW_UPDATE_TARGET"
	chmod +x "${FW_UPDATE_TARGET_PATH}/fullup.sh"
	chmod +x "${FW_UPDATE_TARGET_PATH}/bin/fullup"
	rm -f "${FW_UPDATE_TARGET_PATH}/update-done"

	ccsettingsconsole --advance --rescue_mode_next_boot
	reboot
else
	msg "No software upgrade found. Tried ${SRC_DIR}/${RESCUE_UPDATE_DIR} - ${SRC_DIR}/${FW_UPDATE_DIR}"
	exit 1
fi
