### include guard
(return 0 2>/dev/null)       &&
[[ -z ${FULLUP_DEV_SH_+1} ]] &&
declare -xr FULLUP_DEV_SH_=1 && {
###

# Succeeds iff "$1" is non-empty and does not end in a forward slash. Behaves
# identically in dry run mode.
dev_name_is_valid() {
	[[ "$1" && "${1%/}" == "$1" ]]
}

# In normal mode this function succeeds iff "/dev/$1" exists. In dry run mode
# always succeeds without actually checking anything.
dev_exists() {
	local dev="/dev/$1"
	${dry_eval_}"[[ -e ${dev@Q} ]]"
}

# In normal mode this function succeeds iff "$1" is a valid device name and
# "/dev/$1" exists. In dry run mode the return value is based on the device
# name check alone; see dev_exists() documentation for details.
dev_is_valid() {
	dev_name_is_valid "$1" && dev_exists "$1"
}

# Check if "/dev/$1" is mounted on "/mnt/$1". Behaves identically in dry run
# mode.
dev_is_mounted() {
	local dev
	dev="$(realpath -eq "/dev/$1")" &&
	grep -q "^$dev /mnt/$1/* " /proc/mounts
}

dev_mount() {
	local mnt="/mnt/$1"
	dev_is_mounted "$1" || {
		{ ${dry_}mount -o remount,rw / || true; }
		${dry_}mkdir -p "$mnt" &&
		${dry_}mount "/dev/$1" "$mnt" || {
			return 1
		}
		${dry_:+mount --fake "/dev/$1" "$mnt"}
	} >&2
}

dev_umount() {
	if dev_is_mounted "$1"; then
		${dry_}sync "/dev/$1"
		${dry_}umount "/mnt/$1" || {
			return 1
		}
		${dry_:+umount --fake "/mnt/$1"}
	fi
}

is_ext4_uuid() {
	[[ "$1" =~ ^[[:xdigit:]]{8}(-[[:xdigit:]]{4}){4}[[:xdigit:]]{8}$ ]]
}

dev_mkfs_ext4() {
	local dev="/dev/$1" label="$2"

	# Recycle the current filesystem label iff $2 is undefined. Take
	# the null string as an explicit command to not set a label.
	[[ "${2+x}" ]] || {
		label="$(${dry_}lsblk -no LABEL "$dev")" || {
			return $?
		}
	}

	${dry_}mkfs.ext4 -v ${label:+-L "$label"} \
	                    "$dev" <<< 'y' >&2
}

### include guard
} || ((1))
###
