#!/usr/bin/env bash

# Try to infer the location of this wrapper script. This should work
# fairly reliably if we're currently executing a script file, but in
# any other situation (e.g. if bash is running shellcode from stdin)
# we'll assume it's the current working directory.
d="$(realpath -mqs --relative-base='.' \
              "${BASH_SOURCE[0]:-./.}")"
[[ "${d:0:1}" == '/' ]] || d="$PWD/$d"
d="$(dirname "$d")"

# Parse the platform name from the name of the md5sum file.
p=$(ls -1 "$d/ccpilot-"*".md5sum" 2>/dev/null \
    | sed -e 's,^.*ccpilot-,,' -e 's,\.md5sum$,,')

printf '%s\n' "$@" | grep -Eq '^-([hps]|-help|(T|-target)(=.*)?)$' || {
	case "$(grep -o '^Main\|^Rescue' /etc/issue)" in
	Main) set - --target=rescue "$@";;
	Rescue) set - --target=main "$@";;
	esac
}

set - fullup $p "$@"

p="$d/bin:$PATH"
v=(LEGACY_COMPAT=1 NO_COLOR=1)

{
	printf '%s\n' \
	       '#' \
	       '# This is the fullup.sh backward compatibility wrapper. Do not' \
	       '# use unless absolutely necessary. Behavior is not guaranteed' \
	       '# to be identical with older versions. Only some input argument' \
	       '# combinations are known to behave "close enough" to the legacy' \
	       '# fullup.sh to be useful.' \
	       '#' \
	       '# Will now run this command:' \
	       '('
	printf '\texport PATH=%q\n\t' "$p"
	printf '%q ' "${v[@]}" "$@"
	printf '\n)\n\n'
} 1>&2

export PATH="$p" "${v[@]}"
unset d p v
exec "$@"
